%% Preprocessing 1Ds
%Reference Clendinen, C. S., Lee-McMullen, B., Williams, C., Stupp, G. S., Vandenborne, K., Hahn, D. A., et al. (2014). 13C NMR Metabolomics: Applications at Natural Abundance. Analytical Chemistry. doi:10.1021/ac502346h

spectraC = loadallft; %Carbon
spectraH = loadallft; %Proton

%% class membership 
Y=[0 0 0 0 0 1 1 1 1 1];

%% Reference Proton and Carbon

spectraC=reference(spectraC,0.1,114.8,116.8);

spectraH=reference(spectraH,0.01,0,-0.3);

%% make 2D matrix of 1D spectra
[X,ppm,Titles]=Setup1D(spectraC); %Carbon
figure, plot(ppm,X);

[H,ppmH,Titles]=Setup1D(spectraH); %Proton
figure, plot(ppmH,H);

%% remove unnecessary regions, check alignment
HR=remove_region(H,ppmH,4.70,5.15);
% HR=remove_region(HR,ppmH,-1.00,0.7);
HR=remove_region(HR,ppmH,9.4,12);
figure, plot(ppmH,HR)
set(gca,'XDir','reverse')


%% Align

XAL=star_align1D(X,ppm,'mean','PAFFT',0.08,.5);
figure, plot(ppm,XAL)
set(gca,'XDir','reverse')
title('star alignment')

HAL=star_align1D(H,ppmH,'mean','PAFFT',0.08,.5);
figure, plot(ppmH,HAL)
set(gca,'XDir','reverse')
title('star alignment')


%% Normalize and Scale Full Resolution
%Carbon
XALN=normalize(XAL,'PQN');

XALSN=scale(XALN,'pareto');

figure, plot(ppm,XAL)
set(gca,'XDir','reverse')
title('star alignment')

%Proton
HALN=normalize(HAL,'PQN');

HALSN=scale(HALN,'pareto');

figure, plot(ppmH,HAL)
set(gca,'XDir','reverse')
title('star alignment')


%% Run STOCSY2D

[S,P]=STOCSY2D(HALN,peakmat,ppmH,shifts,'peakPickX',0);

%% Scale Peakpicked Resolution
%proton
peakmatH=S.Hpp;
shiftsH=S.ppmHpp;

plot(peakmatH,shiftsH)
set(gca,'XDir','reverse')

peakmatHSN=scale(peakmatH,'pareto');

%carbon
Xmedian=median(XAL);
peaks=peakpick(Xmedian,30, 0.06 * max(Xmedian));
peakmat=XAL(:,peaks);
shifts=ppm(peaks);

bar(shifts,peakmat(1,:))

peakmatN=normalize(peakmat,'PQN');
peakmatSN=scale(peakmatN,'pareto');


%% Analysis of Carbon

%PCA analysis of peaks for Carbon
PCA_peaks=nipalsPCA(peakmatSN,5);
VisScores(peakmatSN,PCA_peaks,[1 2],Y);
figure; VisLoadings1Dpeak(peakmat,PCA_peaks.loadings(1,:),shifts)





%% Proton

%PCA analysis of peaks for Proton
PCA_peaksH=nipalsPCA(peakmatHSN,5);
VisScores(peakmatHSN,PCA_peaksH,[1 2],Y);
figure; VisLoadings1Dpeak(peakmatH,PCA_peaksH.loadings(1,:),shifts)






%% GenPeaklist
%using only Carbon
 [Clist,PairsX2X]=genPeaklist(S.corrXrr,S.covarXrr,10,S.ppmXrr,S.ppmXrr,'CovarThreshold', 0.4);
% [Clist,PairsX2X]=genPeaklist(S.corrXpp,S.covarXpp,10,S.ppmXpp,S.ppmXpp);

%both Proton and Carbon
[Plist,PairsX2H]=genPeaklist(S.corrHXpp,S.covarHXpp,10,S.ppmHpp,S.ppmXrr);


ppmXpp=S.ppmXrr;
Xpp=median(S.Xrr);

Vis1Dcorr(Xpp(PairsX2X(73).idxH),PairsX2X(73).corrH,PairsX2X(73).covarH,ppmXpp(PairsX2X(73).idxH));
x = PairsX2X(73).toppm;
idxs = findShifts(ppmXpp,x);
figure; Vis1Dcorr(Xpp(idxs),PairsX2X(73).corrsH,PairsX2X(73).covarsH,ppmXpp(idxs));
